/*!
 * \example inetq/inetq.c
 *
 * Requests an URL from the Internet. 
 * Demonstrates DNS query and default route usage.
 */
#define DNSSERVERIP     "192.168.192.2"
#define INETSERVER	    "www.google.com"
#define INETSERVERPORT  80
#define INETURL         "/search?q=˫˾"
#define MY_MAC          {0x12,0x34,0x56,0x00,0x00,0x01}
#define MY_IP           "192.168.192.100"
#define MY_MASK         "255.255.255.0"
#define MY_GATE         "192.168.192.3"

#include <string.h>

#include <dev/uartavr.h>
#include <dev/nicrtl.h>

#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/print.h>
#include <sys/kprint.h>

#include <netinet/sostream.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <netdb.h>

static u_char buff[1024];
static u_char my_mac[] = MY_MAC;

/*
 * Main application routine. 
 *
 * sl/OS automatically calls this entry after initialization.
 */
THREAD(slMain, arg)
{
    NUTDEVICE *uart0;
    u_long baud = 115200;
    TCPSOCKET *sock;
    NUTDEVICE *sostream;
    u_long rip;

    slRegisterDevice(&devUart0, 0, 0);
    uart0 = slDeviceOpen("uart0");
    slDeviceIOCtl(uart0, UART_SETSPEED, &baud);
	slKPrintString("\r\nInetQuery 1.0\r\n");

    /*
    * Register Realtek controller at address 8300 hex
    * and interrupt 5 and configure lan interface.
    */
    slKPrintString("Configuring Ethernet interface\r\n");
    slRegisterDevice(&devEth0, 0x8300, 5);
	{
	    u_long ip = inet_addr(MY_IP);
		u_long msk = inet_addr(MY_MASK);
        slNetIfConfig("eth0", my_mac, ip, msk);
	}
    slIpRouteAdd(0, 0, inet_addr(MY_GATE), &devEth0);
    slDnsConfig(0, 0, inet_addr(DNSSERVERIP));

    if((rip = slDnsGetHostByName(INETSERVER)) != 0) {
    
        /*
         * Create a socket.
         */
        if((sock = slTcpCreateSocket()) != 0) {
            slKPrintFormat("Connecting %s:%u\r\n", inet_ntoa(rip), INETSERVERPORT);
            if(slTcpConnect(sock, rip, INETSERVERPORT) == 0) {
                if((sostream = slSoStreamCreate(sock)) != 0) {
                    slPrintFormat(sostream, "GET %s HTTP/1.0\r\n", INETURL);
                    slPrintString(sostream, "User-Agent: Ethernut/2.3 [en] (slOS)\r\n");
                    slPrintString(sostream, "\r\n");
                    slPrintFlush(sostream);
                    while(slDeviceGetLine(sostream, buff, sizeof(buff) - 1) >= 0) {
                        slKPrintString(buff);
                        slKPrintString("\r\n");
                    }
                    slSoStreamDestroy(sostream);
                }
                else
                    slKPrintString("Creating stream device failed\r\n");

            }
            slKPrintFormat("Disonnecting %s:%u\r\n", inet_ntoa(rip), INETSERVERPORT);
            slTcpCloseSocket(sock);
        }
    }
    else
        slKPrintFormat("Can't resolve %s\r\n", INETSERVER);

    for(;;)
        slSleep(1000);
}

