
#include <eeprom.h>

#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>

#include <sys/confos.h>
#define    RAMEND   0x10FF    //Last On-Chip SRAM Location

CONFOS confos;


extern void slMain(void *arg) __attribute__ ((noreturn));

/*! \fn slIdle(void *arg)
 * \brief Idle thread. 
 */
THREAD(slIdle, arg)
{
    slTimerInit();

    slThreadCreate("main", slMain, 0, 768);
    slThreadSetPriority(255);
    for(;;) {
        slThreadYield();
    }
}

/*!
 * \brief Load sl/OS configuration from the EEPROM.
 *
 * This routine is automatically called during system
 * initialization.
 *
 * \return 0 if OK, -1 if configuration isn't available.
 */
int slLoadConfig(void)
{
    eeprom_read_block(&confos, CONFOS_EE_OFFSET, sizeof(CONFOS));
    if(confos.size != sizeof(CONFOS) || confos.magic[0] != 'O' || confos.magic[1] != 'S') {
        return -1;
    }
    return 0;
}

/*!
 * \brief Save sl/OS configuration in the EEPROM.
 *
 * \return 0 if OK, -1 on failures.
 */
int slSaveConfig(void)
{
    u_char *cp;
    int i;

    confos.size = sizeof(CONFOS);
    confos.magic[0] = 'O';
    confos.magic[1] = 'S';
    for(cp = (u_char *)&confos, i = 0; i < sizeof(CONFOS); i++)
        eeprom_wb(i + CONFOS_EE_OFFSET, *cp++);

    return 0;
}

/*!
 * \brief sl/OS main entry.
 *
 * This is the entry point for C programs. To hide neccessary
 * system initialization from the user program, this entry point
 * is included in the sl/OS library as a separate object module.
 *
 * It will initialize memory management and the thread system
 * and start an idle thread, which in turn initializes the
 * timer functions. Finally slMain() is called, which must
 * be defined by the application and may be used to start
 * additional application threads.
 * \internal
 */
int main(void)
{

    /*
     * Read eeprom configuration.
     */
    if(slLoadConfig())
        slSaveConfig();

    outp(BV(SRE) | BV(SRW), MCUCR);

    if((short)RAMEND - (short)(&__bss_end) > 384)
        slHeapAdd(&__bss_end, (u_short)RAMEND - 256 - (u_short)(&__bss_end));
    slHeapAdd((void *)(RAMEND + 1), 0x7FFF - RAMEND);
    
    slThreadCreate("idle", slIdle, 0, 384);

    return 0;
}

/*@}*/
