/*!
 * \example tcps/tcps.c
 *
 * Simple TCP server.
 *
 * Program Ethernut with tcps.rom and enter
 *
 * telnet x.x.x.x 12345
 *
 * on a command prompt, replacing x.x.x.x with the
 * IP address of your ethernut board. Enter help
 * for a list of available commands.
 */

#include <string.h>

#include <dev/nicrtl.h>

#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/print.h>

#include <netinet/sostream.h>
#include <arpa/inet.h>

static u_char buff[128];

/*
 * Process client requests.
 */
void ProcessRequests(NUTDEVICE *sostream)
{
    int l;

    strcmp(buff, "TEST");
    slPrintString(sostream, "200 Welcome to tcps. Type help to get help.\r\n");
    for(;;) {
        slPrintFlush(sostream);
        if(slDeviceGetLine(sostream, buff, sizeof(buff)) < 0)
            break;
        /*
         * Ignore blank lines.
         */
        if(buff[0] == 0)
            continue;

        /*
         * Memory info.
         */
        l = strlen(buff);
        if(strncmp(buff, "memory", l) == 0) {
            slPrintFormat(sostream, "210 %u bytes RAM free\r\n", slHeapAvailable());
            continue;
        }

        /*
         * List threads.
         */
        if(strncmp(buff, "threads", l) == 0) {
            NUTTHREADINFO *tdp;
            NUTTIMERINFO *tnp;

            slPrintString(sostream, "220 List of threads with name,state,prio,stack,mem,timeout follows\r\n");
            for(tdp = nutThreadList; tdp; tdp = tdp->td_next) {
                slPrintString(sostream, tdp->td_name);
                slPrintString(sostream, "\t");
                switch(tdp->td_state) {
                case TDS_TERM:
                    slPrintString(sostream, "\tTerm\t");
                    break;
                case TDS_RUNNING:
                    slPrintString(sostream, "\tRun\t");
                    break;
                case TDS_READY:
                    slPrintString(sostream, "\tReady\t");
                    break;
                case TDS_SLEEP:
                    slPrintString(sostream, "\tSleep\t");
                    break;
                }
                slPrintInteger(sostream, tdp->td_priority, 10, 0, 0);
                slPrintString(sostream, "\t");
                slPrintInteger(sostream, (u_short)tdp->td_sp - (u_short)tdp->td_memory, 10, 0, 0);
                if(*((u_long *)tdp->td_memory) != DEADBEEF)
                    slPrintString(sostream, "\tCorrupted\t");
                else
                    slPrintString(sostream, "\tOK\t");

                if((tnp = (NUTTIMERINFO *)tdp->td_timer) != 0) 
                    slPrintInteger(sostream, tnp->tn_ticks_left, 10, 0, 0);
                else
                    slPrintString(sostream, "None");
                slPrintString(sostream, "\r\n");
            }
            slPrintString(sostream, ".\r\n");
            continue;
        }

        /*
         * List threads.
         */
        if(strncmp("timers", buff, l) == 0) {
            NUTTIMERINFO *tnp;

            slPrintString(sostream, "221 List of timers with ticks left and interval follows\r\n");
            for(tnp = nutTimerList; tnp; tnp = tnp->tn_next) {
                slPrintInteger(sostream, tnp->tn_ticks_left, 10, 0, 0);
                if(tnp->tn_ticks) {
                    slPrintString(sostream, "\t");
                    slPrintInteger(sostream, tnp->tn_ticks, 10, 0, 0);
                }
                else 
                    slPrintString(sostream, "\tOneshot");
                slPrintString(sostream, "\r\n");
            }
            slPrintString(sostream, ".\r\n");
            continue;
        }
        slPrintString(sostream, "400 List of commands follows\r\n");
        slPrintString(sostream, "memory\tQueries number of RAM bytes free\r\n");
        slPrintString(sostream, "threads\tLists all created threads\r\n");
        slPrintString(sostream, "timers\tLists all running timers\r\n");
        slPrintString(sostream, ".\r\n");
    }
}

/*
 * Main application routine. 
 *
 * sl/OS automatically calls this entry after initialization.
 */
THREAD(slMain, arg)
{
    TCPSOCKET *sock;
    NUTDEVICE *sostream;

    /*
     * Register Realtek controller at address 8300 hex
     * and interrupt 5.
     */
    slRegisterDevice(&devEth0, 0x8300, 5);

    /*
     * Configure lan interface. 
     *
     */
    slNetAutoConfig("eth0");
    
    /*
     * Now loop endless for connections.
     */
    for(;;) {
        /*
         * Create a socket.
         */
        sock = slTcpCreateSocket();

        /*
         * Listen on port 12345. If we return,
         * we got a client.
         */
        slTcpAccept(sock, 12345);

        /*
         * Create a device from the socket,
         * so we can use slPrintFormat() and
         * other device goodies.
         */
        sostream = slSoStreamCreate(sock);

        /*
         * Process client requests.
         */
        ProcessRequests(sostream);

        /*
         * Destroy our device.
         */
        slSoStreamDestroy(sostream);

        /*
         * Close our socket.
         */
        slTcpCloseSocket(sock);
    }
}
