/*!
 * \example threads/threads.c
 *
 * This sample demonstrates sl/OS multithreading.
 */

#include <dev/uartavr.h>
#include <sys/thread.h>
#include <sys/print.h>
#include <sys/timer.h>

static NUTDEVICE *uart;

/*
 * High priority thread.
 */
THREAD(Thread1, arg)
{
    /*
     * Endless loop in high priority thread.
     */
    slThreadSetPriority(16);
    for(;;) {
        slPrintString(uart, "H");
        slPrintFlush(uart);
        slSleep(200);
    }
}

/*
 * Low priority thread.
 */
THREAD(Thread2, arg)
{
    /*
     * Endless loop in low priority thread.
     */
    slThreadSetPriority(128);
    for(;;) {
        slPrintString(uart, "L");
        slPrintFlush(uart);
        slSleep(1000);
    }
}

/*
 * Main application thread. 
 */
THREAD(slMain, arg)
{
    u_long baud = 115200;

    /*
     * Register the UART device, open it and
     * set the baudrate.
     */
    slRegisterDevice(&devUart0, 0, 0);
    uart = slDeviceOpen("uart0");
    slDeviceIOCtl(uart, UART_SETSPEED, &baud);

    slPrintString(uart, "TimerTest\r\n");
    slPrintFlush(uart);

    /*
     * Start two additional threads. All threads
     * are started with priority 64.
     */
    slThreadCreate("t1", Thread1, 0, 1024);
    slThreadCreate("t2", Thread2, 0, 1024);

    /*
     * Endless loop in main thread.
     */
    for(;;) {
        slPrintString(uart, "M");
        slPrintFlush(uart);
        slSleep(500);
    }
}

