
#ifdef __IMAGECRAFT__

#ifdef _MCU_enhanced
#include <iom128.h>
#else
#include <iom103.h>
#endif

#else

#include <interrupt.h>
#include <sig-avr.h>

#endif

#include <sys/atom.h>
#include <dev/irqreg.h>

/*
 * Registered interrupt handler information structure.
 */
typedef struct {
    u_long ir_count;
    void *ir_arg;
    void (*ir_handler)(void *);
} IRQ_HANDLER;

static IRQ_HANDLER irqHandler[IRQ_MAX];

static void CallHandler(IRQ_HANDLER *irh)
{
    irh->ir_count++;
    if(irh->ir_handler)
	    (irh->ir_handler)(irh->ir_arg);    	   	
}
    
/*! 
 * \fn SIG_INTERRUPT0(void)
 * \brief External interrupt 0 entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INTERRUPT0:IT_INT0
#endif
SIGNAL(SIG_INTERRUPT0)
{
    CallHandler(&irqHandler[IRQ_INT0]);
}

/*! \fn SIG_INTERRUPT1(void)
 * \brief External interrupt 1 entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INTERRUPT1:IT_INT1
#endif
SIGNAL(SIG_INTERRUPT1)
{
    CallHandler(&irqHandler[IRQ_INT1]);
}

/*! \fn SIG_INTERRUPT2(void)
 * \brief External interrupt 2 entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INTERRUPT2:IT_INT2
#endif
SIGNAL(SIG_INTERRUPT2)
{
    CallHandler(&irqHandler[IRQ_INT2]);
}

/*! \fn SIG_INTERRUPT3(void)
 * \brief External interrupt 3 entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INTERRUPT3:IT_INT3
#endif
SIGNAL(SIG_INTERRUPT3)
{
    CallHandler(&irqHandler[IRQ_INT3]);
}

/*! \fn SIG_INTERRUPT4(void)
 * \brief External interrupt 4 entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INTERRUPT4:IT_INT4
#endif
SIGNAL(SIG_INTERRUPT4)
{
    CallHandler(&irqHandler[IRQ_INT4]);
}

/*! \fn SIG_INTERRUPT5(void)
 * \brief External interrupt 5 entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INTERRUPT5:IT_INT5
#endif
SIGNAL(SIG_INTERRUPT5)
{
    CallHandler(&irqHandler[IRQ_INT5]);
}

/*! \fn SIG_INTERRUPT6(void)
 * \brief External interrupt 6 entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INTERRUPT6:IT_INT6
#endif
SIGNAL(SIG_INTERRUPT6)
{
    CallHandler(&irqHandler[IRQ_INT6]);
}

/*! \fn SIG_INTERRUPT7(void)
 * \brief External interrupt 7 entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INTERRUPT7:IT_INT7
#endif
SIGNAL(SIG_INTERRUPT7)
{
    CallHandler(&irqHandler[IRQ_INT7]);
}

/*! \fn SIG_OUTPUT_COMPARE2(void) 
 * \brief Timer 2 output compare interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OUTPUT_COMPARE2:IT_TIMER2_COMP
#endif
SIGNAL(SIG_OUTPUT_COMPARE2)
{
    CallHandler(&irqHandler[IRQ_TIMER2_COMP]);
}

/*! \fn SIG_OVERFLOW2(void)
 * \brief Timer 2 overflow interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OVERFLOW2:IT_TIMER2_OVF
#endif
SIGNAL(SIG_OVERFLOW2)
{
    CallHandler(&irqHandler[IRQ_TIMER2_OVF]);
}

/*! \fn SIG_INPUT_CAPTURE1(void)
 * \brief Timer 1 input capture interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INPUT_CAPTURE1:IT_TIMER1_CAPT
#endif
SIGNAL(SIG_INPUT_CAPTURE1)
{
    CallHandler(&irqHandler[IRQ_TIMER1_CAPT]);
}

/*! \fn SIG_OUTPUT_COMPARE1A(void)
 * \brief Timer 1A output compare interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OUTPUT_COMPARE1A:IT_TIMER1_COMPA
#endif
SIGNAL(SIG_OUTPUT_COMPARE1A)
{
    CallHandler(&irqHandler[IRQ_TIMER1_COMPA]);
}

/*! \fn SIG_OUTPUT_COMPARE1B(void)
 * \brief Timer 1B output compare interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OUTPUT_COMPARE1B:IT_TIMER1_COMPB
#endif
SIGNAL(SIG_OUTPUT_COMPARE1B)
{
    CallHandler(&irqHandler[IRQ_TIMER1_COMPB]);
}

/*! \fn SIG_OVERFLOW1(void)
 * \brief Timer 1 overflow interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OVERFLOW1:IT_TIMER1_OVF
#endif
SIGNAL(SIG_OVERFLOW1)
{
    CallHandler(&irqHandler[IRQ_TIMER1_OVF]);
}

/*! \fn SIG_OUTPUT_COMPARE0(void)
 * \brief Timer 0 output compare interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OUTPUT_COMPARE0:IT_TIMER0_COMP
#endif
SIGNAL(SIG_OUTPUT_COMPARE0)
{
    CallHandler(&irqHandler[IRQ_TIMER0_COMP]);
}

/*! \fn SIG_OVERFLOW0(void)
 * \brief Timer 0 overflow interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OVERFLOW0:IT_TIMER0_OVF
#endif
SIGNAL(SIG_OVERFLOW0)
{
    CallHandler(&irqHandler[IRQ_TIMER0_OVF]);
}

/*! \fn SIG_SPI(void)
 * \brief SPI interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_SPI:IT_SPI_STC
#endif
SIGNAL(SIG_SPI)
{
    CallHandler(&irqHandler[IRQ_SPI_STC]);
}

#ifdef __AVR_ATmega128__

/*! \fn SIG_UART0_TRANS(void)
 * \brief Uart0 transmit complete interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART0_TRANS:IT_USART0_TX
#endif
SIGNAL(SIG_UART0_TRANS)
{
    CallHandler(&irqHandler[IRQ_UART_TX]);
}

/*! \fn SIG_UART0_DATA(void)
 * \brief Uart0 data register empty interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART0_DATA:IT_USART0_UDRE
#endif
SIGNAL(SIG_UART0_DATA)
{
    CallHandler(&irqHandler[IRQ_UART_UDRE]);
}

/*! \fn SIG_UART0_RECV(void)
 * \brief Uart0 receive complete interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART0_RECV:IT_USART0_RX
#endif
SIGNAL(SIG_UART0_RECV)
{
    CallHandler(&irqHandler[IRQ_UART_RX]);
}

#else

/*! \fn SIG_UART_TRANS(void)
 * \brief Uart0 transmit complete interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART_TRANS:IT_UART_TX
#endif
SIGNAL(SIG_UART_TRANS)
{
    CallHandler(&irqHandler[IRQ_UART_TX]);
}

/*! \fn SIG_UART_DATA(void)
 * \brief Uart0 data register empty interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART_DATA:IT_UART_UDRE
#endif
SIGNAL(SIG_UART_DATA)
{
    CallHandler(&irqHandler[IRQ_UART_UDRE]);
}

/*! \fn SIG_UART_RECV(void)
 * \brief Uart0 receive complete interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART_RECV:IT_UART_RX
#endif
SIGNAL(SIG_UART_RECV)
{
    CallHandler(&irqHandler[IRQ_UART_RX]);
}

#endif

/*! \fn SIG_ADC(void)
 * \brief ADC conversion complete interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_ADC:IT_ADC
#endif
SIGNAL(SIG_ADC)
{
    CallHandler(&irqHandler[IRQ_ADC]);
}

#ifdef __AVR_ATmega128__

/*! \fn SIG_EEPROM_READY(void)
 * \brief EEPROM ready interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_EEPROM_READY:IT_EE_READY
#endif
SIGNAL(SIG_EEPROM_READY)
{
    CallHandler(&irqHandler[IRQ_EE_RDY]);
}

#else

/*! \fn SIG_EEPROM(void)
 * \brief EEPROM ready interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_EEPROM:IT_EE_READY
#endif
SIGNAL(SIG_EEPROM)
{
    CallHandler(&irqHandler[IRQ_EE_RDY]);
}

#endif

/*! \fn SIG_COMPARATOR(void)
 * \brief Analog comparator interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_COMPARATOR:IT_ANALOG_COMP
#endif
SIGNAL(SIG_COMPARATOR)
{
    CallHandler(&irqHandler[IRQ_ANA_COMP]);
}

#ifdef __AVR_ATmega128__

/*! \fn SIG_OUTPUT_COMPARE1C(void)
 * \brief Timer 1C output compare interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OUTPUT_COMPARE1C:IT_TIMER1_COMPC
#endif
SIGNAL(SIG_OUTPUT_COMPARE1C)
{    
    CallHandler(&irqHandler[IRQ_TIMER1_COMPC]);
}

/*! \fn SIG_INPUT_CAPTURE3(void)
 * \brief Timer 3 input capture interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_INPUT_CAPTURE3:IT_TIMER3_CAPT
#endif
SIGNAL(SIG_INPUT_CAPTURE3)
{    
    CallHandler(&irqHandler[IRQ_TIMER3_CAP]);
}

/*! \fn SIG_OUTPUT_COMPARE3A(void)
 * \brief Timer 3A output compare interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OUTPUT_COMPARE3A:IT_TIMER3_COMPA
#endif
SIGNAL(SIG_OUTPUT_COMPARE3A)
{
    CallHandler(&irqHandler[IRQ_TIMER3_COMPA]);
}

/*! \fn SIG_OUTPUT_COMPARE3B(void)
 * \brief Timer 3B output compare interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OUTPUT_COMPARE3B:IT_TIMER3_COMPB
#endif
SIGNAL(SIG_OUTPUT_COMPARE3B)
{
    CallHandler(&irqHandler[IRQ_TIMER3_COMPB]);
}

/*! \fn SIG_OUTPUT_COMPARE3C(void)
 * \brief Timer 3C output compare interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OUTPUT_COMPARE3C:IT_TIMER3_COMPC
#endif
SIGNAL(SIG_OUTPUT_COMPARE3C)
{
    CallHandler(&irqHandler[IRQ_TIMER3_COMPC]);
}

/*! \fn SIG_OVERFLOW3(void)
 * \brief Timer 3 overflow interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_OVERFLOW3:IT_TIMER3_OVF
#endif
SIGNAL(SIG_OVERFLOW3)
{
    CallHandler(&irqHandler[IRQ_TIMER3_OVF]);
}

/*! \fn SIG_UART1_RECV(void)
 * Uart1 receive complete interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART1_RECV:IT_USART1_RX
#endif
SIGNAL(SIG_UART1_RECV)
{
    CallHandler(&irqHandler[IRQ_UART1_RX]);
}

/*! \fn SIG_UART1_DATA(void)
 * Uart1 data register empty interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART1_DATA:IT_USART1_UDRE
#endif
SIGNAL(SIG_UART1_DATA)
{
    CallHandler(&irqHandler[IRQ_UART1_UDRE]);
}

/*! \fn SIG_UART1_TRANS(void)
 * Uart1 transmit complete interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_UART1_TRANS:IT_USART1_TX
#endif
SIGNAL(SIG_UART1_TRANS)
{
    CallHandler(&irqHandler[IRQ_UART1_TX]);
}


/*! \fn SIG_2WIRE_SERIAL(void)
 * \brief Two-wire serial interface interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_2WIRE_SERIAL:IT_TWI
#endif
SIGNAL(SIG_2WIRE_SERIAL)
{
    CallHandler(&irqHandler[IRQ_I2C]);
}

/*! \fn SIG_SPM_READY(void)
 * \brief Store program memory interrupt entry.
 */
#ifdef __IMAGECRAFT__
#pragma interrupt_handler SIG_SPM_READY:IT_SPM_READY
#endif
SIGNAL(SIG_SPM_READY)
{
    CallHandler(&irqHandler[IRQ_SPM_RDY]);
}

#endif

/*!
 * \brief Register an interrupt handler.
 *
 * This function is typically called by device drivers.
 *
 * \param irq     Interrupt number to be associated with this handler.
 * \param handler This routine will be called by sl/OS, when the
 *                specified interrupt occurs.
 * \param arg     Argument to be passed to the interrupt handler.
 *
 * \return 0 on success, -1 otherwise.
 */
int slRegisterInterrupt(int irq, void (*handler)(void *), void *arg)
{
    slEnterCritical();

    irqHandler[irq].ir_arg = arg;
    irqHandler[irq].ir_handler = handler;

    slExitCritical();
    return 0;
}

/*@}*/
