#ifndef _NET_NETBUF_H_
#define _NET_NETBUF_H_


#include <sys/types.h>

/*!
 * \file dev/netbuf.h
 * \brief Network buffer definitions.
 */


/* lower bits may be used as ref-count, if ever needed */
#define NBAF_DATALINK    0x10   /*!< \brief Datalink buffer allocated flag. */
#define NBAF_NETWORK     0x20   /*!< \brief Network buffer allocated flag. */
#define NBAF_TRANSPORT   0x40   /*!< \brief Transport buffer allocated flag. */
#define NBAF_APPLICATION 0x80   /*!< \brief Application buffer allocated flag. */
#define NBAF_ALL         0xf0   /*!< \brief Masks allocated buffer flags flag. */

/*!
 * \brief Network buffer type.
 */
typedef struct _NETBUF NETBUF;

/*!
 * \brief Network buffer data type.
 */
typedef struct _NBDATA NBDATA;

/*!
 * \struct _NBDATA netbuf.h dev/netbuf.h
 * \brief Data part of a network buffer structure.
 */
struct _NBDATA {
    void   *vp; /*!< \brief Pointer to data buffer. */
    u_short sz; /*!< \brief Size of data buffer. */
};

/*!
 * \struct _NETBUF netbuf.h dev/netbuf.h
 * \brief Network buffer structure.
 */
struct _NETBUF {
    NETBUF *nb_next;    /*!< \brief Link to next structure. */
    u_char nb_flags;    /*!< \brief NBAF_ flags. */
    NBDATA nb_dl;       /*!< \brief Datalink buffer. */
    NBDATA nb_nw;       /*!< \brief Network buffer. */
    NBDATA nb_tp;       /*!< \brief Transport buffer. */
    NBDATA nb_ap;       /*!< \brief Application buffer. */
};

/*@}*/

extern NETBUF *slNetBufAlloc(NETBUF *nb, u_char type, u_short size);
extern NETBUF *slNetBufClone(NETBUF *nb);
extern int slNetBufFree(NETBUF *nb);

#endif
