#ifndef _DEV_NICRTL_H_
#define _DEV_NICRTL_H_



#include <dev/netbuf.h>
#include <net/if_var.h>


/*!
 * \file dev/nicrtl.h
 * \brief Network interface controller definitions.
 */

/*!
 * \struct _NICINFO nicrtl.h dev/nicrtl.h
 * \brief Network interface controller information structure.
 */
struct _NICINFO {
    u_char volatile ni_tx_bsy;      /*!< NIC transmitter busy flags. */
    HANDLE volatile ni_rx_rdy;      /*!< Receiver event queue. */
    HANDLE volatile ni_tx_rdy;      /*!< Transmitter event queue. */
    NETBUF * volatile ni_tx_nbq;    /*!< Queued packets waiting for transmission. */
    u_short ni_tx_cnt;              /*!< Number of bytes in transmission queue. */
    u_char ni_curr_page;            /*!< Current receive page. */
    u_long ni_rx_packets;           /*!< Number of packets received. */
    u_long ni_tx_packets;           /*!< Number of packets sent. */
    u_long ni_interrupts;           /*!< Number of interrupts. */
    u_long ni_overruns;             /*!< Number of packet overruns. */
    u_long ni_rx_frame_errors;      /*!< Number of frame errors. */
    u_long ni_rx_crc_errors;        /*!< Number of CRC errors. */
    u_long ni_rx_missed_errors;     /*!< Number of missed packets. */
};

/*!
 * \brief Network interface controller information type.
 */
typedef struct _NICINFO NICINFO;



/*
 * Available drivers.
 */
extern NUTDEVICE devEth0;

/*
 * Driver routines.
 */
extern int NicInit(NUTDEVICE *dev);
extern int NicOutput(NUTDEVICE *dev, NETBUF *nb);

#endif
