#ifndef _DEV_UARTAVR_H_
#define _DEV_UARTAVR_H_


#include <sys/device.h>

/*!
 * \file dev/uartavr.h
 * \brief AVR on-chip UART definitions.
 */


/*!
 * UART device control block type.
 */
typedef struct _UARTDCB UARTDCB;

/*!
 * \struct _UARTDCB uartavr.h dev/uartavr.h
 * \brief UART device control block structure.
 */
struct _UARTDCB {
    /*! \brief Baudrate selector.
     *
     * See the ATmega datasheet for further details.
     */
    u_char dcb_baudSelect;

    /*! \brief Read timeout.
     */
    u_long dcb_rtimeout;

    /*! \brief Write timeout.
     */
    u_long dcb_wtimeout;

    /*! \brief Queue of threads waiting for output buffer empty.
     *
     * Threads are added to this queue when calling UartAvrFlush(). 
     */
    HANDLE dcb_tx_rdy;

    /*! \brief Queue of threads waiting for a character in the input buffer.
     *
     * Threads are added to this queue when calling UartAvrInput(). 
     */
    HANDLE dcb_rx_rdy;
};

/*@}*/

/*
 * Available devices.
 */
extern NUTDEVICE devUart0;
#ifdef __AVR_ATmega128__
    extern NUTDEVICE devUart1;
#endif

#endif
