#ifndef _DEV_UARTSPI_H_
#define _DEV_UARTSPI_H_


#include <sys/device.h>

/*!
 * \file dev/uartspi.h
 * \brief SPI UART definitions.
 */


/*!
 * UART device control block type.
 */
typedef struct _UARTSDCB UARTSDCB;

/*!
 * \struct _UARTSDCB uartspi.h dev/uartspi.h
 * \brief UART device control block structure.
 */
struct _UARTSDCB {
    /*! \brief Baudrate selector.
     *
     * See devices datasheet for further details.
     */
    u_char dcb_baudSelect;

    /*! \brief Mode.
     *  - xxxx xxx0  8 bit
     *  - xxxx xxx1  7 bit
     *  - xxxx x0xx  No parity bit
     *  - xxxx x10x  Even parity
     *  - xxxx x11x  Odd parity
     */
    u_char dcb_mode;

    /*! \brief Read timeout.
     */
    u_long dcb_rtimeout;

    /*! \brief Write timeout.
     */
    u_long dcb_wtimeout;

    /*! \brief Configuration flag.
     * Set if device has been configured.
     */
    volatile u_char dcb_configured;

    /*! \brief Configuration flag.
     * Set if device has been configured.
     */
    u_long dcb_crystal;

    /*! \brief Device firmware version.
     */
    u_long dcb_version;

    /*! \brief Queue of threads waiting for output buffer empty.
     *
     * Threads are added to this queue when calling UartSpiFlush(). 
     */
    HANDLE dcb_tx_rdy;

    /*! \brief Queue of threads waiting for a character in the input buffer.
     *
     * Threads are added to this queue when calling UartSpiInput(). 
     */
    HANDLE dcb_rx_rdy;
};

/*@}*/

/*
 * Available drivers.
 */
extern NUTDEVICE devUarts[];

extern int UartSpiInit(NUTDEVICE *dev);
extern int UartSpiIOCtl(NUTDEVICE *dev, int req, void *conf);


#endif
