#ifndef _FS_UROMFS_H_
#define _FS_UROMFS_H_


#include <pgmspace.h>
#include <sys/types.h>

/*!
 * \file fs/uromfs.h
 * \brief Micro-ROM file system definitions.
 */

/*!
 * \brief Mikro-ROM directory entry type.
 */
typedef struct _ROMENTRY ROMENTRY;

/*!
 * \struct _ROMENTRY uromfs.h fs/uromfs.h
 * \brief Mikro-ROM directory entry structure.
 */
#ifdef __IMAGECRAFT__
struct _ROMENTRY {
    ROMENTRY *rome_next;    /*!< Link to next ROMENTRY structure. */
    u_char   *rome_name;    /*!< Filename. */
    u_short   rome_size;    /*!< File size. */
    const char *rome_data;  /*!< File data. */
};
#else
struct _ROMENTRY {
    ROMENTRY *rome_next;    /*!< Link to next ROMENTRY structure. */
    u_char   *rome_name;    /*!< Filename. */
    u_short   rome_size;    /*!< File size. */
    prog_char *rome_data; /* __attribute__ ((progmem));  !< File data. */
};
#endif

/*!
 * \brief Mikro-ROM file information type.
 */
typedef struct _ROMFILE ROMFILE;

/*!
 * \struct _ROMFILE uromfs.h fs/uromfs.h
 * \brief Mikro-ROM file information structure.
 */
struct _ROMFILE {
    ROMENTRY *romf_entry;   /*!< Points to ROMENTRY */
    u_short  romf_pos;      /*!< Current read position. */
};

/*!
 * \brief Linked list of all microROM files.
 */
extern ROMENTRY *romEntryList;

extern ROMFILE *slRomFileOpen(char *name);
extern int slRomFileClose(ROMFILE *romf);
extern int slRomFileRead(ROMFILE *romf, void *data, int size);
extern int slRomFileSize(ROMFILE *romf);
extern int slRomFileSeek(ROMFILE *romf, int pos);

#endif
