#ifndef _NET_IF_ARP_H_
#define _NET_IF_ARP_H_


#include <sys/types.h>

/*!
 * \file net/if_arp.h
 * \brief ARP protocol header.
 *
 * See RFC 826 for protocol description.
 */

#define ARPHRD_ETHER    1   /*!< \brief ethernet hardware format */

#define ARPOP_REQUEST   1   /*!< \brief request to resolve address */
#define ARPOP_REPLY     2   /*!< \brief response to previous request */

/*!
 * \struct arphdr if_arp.h net/if_arp.h
 * \brief ARP packet header structure.
 */
/*!
 * \typedef ARPHDR
 * \brief ARP packet header type.
 */
typedef struct arphdr {
    u_short ar_hrd;    /*!< \brief Format of hardware address. 
                        *   sl/Net supports ARPHRD_ETHER only. 
                        */
    u_short ar_pro;    /*!< \brief Format of protocol address.
                        *   Only ETHERTYPE_IP is supported by slos/Net.
                        */
    u_char  ar_hln;    /*!< \brief Length of hardware address. */
    u_char  ar_pln;    /*!< \brief Length of protocol address. */
    u_short ar_op;     /*!< \brief ARP operation.
                        *   May be either ARPOP_REQUEST or ARPOP_REPLY.
                        */
} ARPHDR;

#endif
