#ifndef _NET_IF_VAR_H_
#define _NET_IF_VAR_H_



#include <sys/types.h>
#include <sys/device.h>
#include <dev/netbuf.h>
#include <net/if_types.h>

/*!
 * \file net/if_var.h
 * \brief Network interface structure.
 *
 * Each network device in sl/Net has an associated
 * network interface structure.
 */

/*!
 * \addtogroup xgIP
 */
/*@{*/

#define ATF_COM     0x02        /*!< \brief Completed entry. */
#define ATF_PERM    0x04        /*!< \brief Permanent entry. */

/*!
 * \brief ARP entry type.
 */
typedef struct _ARPENTRY ARPENTRY;

/*!
 * \struct _ARPENTRY if_var.h net/if_var.h
 * \brief ARP entry structure.
 *
 * Each network interface maintains its own ARP table.
 */
struct _ARPENTRY {
    ARPENTRY *ae_next;  /*!< \brief Linked list chain. */
    u_long ae_ip;       /*!< \brief IP address. */
    u_char ae_ha[6];    /*!< \brief Hardware address. */
    u_char ae_flags;    /*!< \brief Status flags, permanent and completed. */
    u_char ae_outdated; /*!< \brief Minutes since last use. */
    HANDLE ae_tq;       /*!< \brief Threads waiting for entry to be completed. */
};

/*!
 * \brief Network interface type.
 */
typedef struct ifnet IFNET;

/*!
 * \struct ifnet if_var.h net/if_var.h
 * \brief Network interface structure.
 *
 * Contains information about the network interface.
 */
struct ifnet {
    u_char  if_type;        /*!< \brief Interface type.
                             *   sl/Net supports IFT_ETHER only.
                             */
    u_char  if_mac[6];      /*!< \brief Hardware net address. */
    u_long  if_local_ip;    /*!< \brief IP address. */
    u_long  if_remote_ip;   /*!< \brief Remote IP address for point to point. */
    u_long  if_mask;        /*!< \brief IP network mask. */
    u_short if_mtu;         /*!< \brief Maximum size of a transmission unit. */
    u_short if_pkt_id;      /*!< \brief Packet identifier. */
    ARPENTRY *arpTable;     /*!< \brief Linked list of arp entries. */
    void (*if_recv)(NUTDEVICE *, NETBUF *); /*!< \brief Receive routine. */
    int  (*if_send)(NUTDEVICE *, NETBUF *); /*!< \brief Send routine. */
};

extern int slNetIfConfig(CONST char *name, CONST u_char *mac, u_long ip_addr, u_long ip_mask);
extern int slNetIfSetup(NUTDEVICE *dev, u_long ip_addr, u_long ip_mask, u_long gateway);

extern int slNetLoadConfig(CONST char *name);
extern int slNetSaveConfig(void);
extern int slNetAutoConfig(CONST char *name);

/*@}*/

#endif
