#ifndef _NETINET_IF_ETHER_H_
#define _NETINET_IF_ETHER_H_


#include <sys/types.h>
#include <net/if_var.h>
#include <net/if_arp.h>

/*!
 * \file netinet/if_ether.h
 * \brief Ethernet interface definitions.
 *
 * A more elaborated file description.
 */

/*!
 * \struct ether_header if_ether.h netinet/if_ether.h
 * \brief Ethernet protocol header.
 */
/*!
 * \typedef ETHERHDR
 * \brief Ethernet protocol header type.
 */
typedef struct ether_header {
    u_char  ether_dhost[6];     /*!< \brief Destination MAC address. */
    u_char  ether_shost[6];     /*!< \brief Source MAC address. */
    u_short ether_type;         /*!< \brief Protocol type. */
} ETHERHDR;

#define ETHERTYPE_IP    0x0800  /*!< \brief IP protocol */
#define ETHERTYPE_ARP   0x0806  /*!< \brief Address resolution protocol */

#define ETHERMTU    1500        /*!< \brief Ethernet maximum transfer unit. */
//#define ETHERMTU    567           /*!< \brief Ethernet maximum transfer unit. */
#define ETHERMIN    (60-14)     /*!< \brief Ethernet minimum transfer unit. */

/*!
 * \struct ether_arp if_ether.h netinet/if_ether.h
 * \brief Ethernet ARP protocol structure.
 *
 * See RFC 826 for protocol description.
 */
/*!
 * \typedef ETHERARP
 * \brief Ethernet ARP protocol type.
 */
typedef struct ether_arp {
    ARPHDR ea_hdr;     /*!< \brief Fixed-size header. */
    u_char arp_sha[6]; /*!< \brief Source hardware address. */
    u_long arp_spa;    /*!< \brief Source protocol address. */
    u_char arp_tha[6]; /*!< \brief Target hardware address. */
    u_long arp_tpa;    /*!< \brief Target protocol address. */
} ETHERARP;

extern void slArpInput(NUTDEVICE *dev, NETBUF *nb);

extern NETBUF *slArpAllocNetBuf(u_short type, u_long ip, u_char *mac);
extern int slArpOutput(NUTDEVICE *dev, NETBUF *nb);

extern void slArpCacheUpdate(NUTDEVICE *dev, u_long ip, u_char *ha);

extern int slArpCacheQuery(NUTDEVICE *dev, u_long ip, u_char *mac);


#endif
