#ifndef _NETINET_IP_H_
#define _NETINET_IP_H_


#include <net/if_arp.h>

/*!
 * \file netinet/ip.h
 * \brief Definitions for internet protocol version 4.
 *
 * RFC 791.
 */

/*!
 * \addtogroup xgIP
 */
/*@{*/

#define IPVERSION   4   /*!< \brief IP protocol version. */

#define IP_DF       0x4000  /*!< \brief Don't fragment flag. */
#define IP_MF       0x2000  /*!< \brief More fragments flag. */
#define IP_OFFMASK  0x1fff  /*!< \brief Mask for fragmenting bits. */

/*!
 * \brief Internet header type.
 */
typedef struct ip IPHDR;

/*!
 * \struct ip ip.h netinet/ip.h
 * \brief Structure of an internet header.
 */
struct ip {
#ifdef __IMAGECRAFT__
    unsigned ip_hl:4,
             ip_v:4; 
#else
    u_char  ip_hl:4,        /*!< \brief Header length. */
            ip_v:4;         /*!< \brief Version. */
#endif
    u_char  ip_tos;         /*!< \brief Type of service. */
    short   ip_len;         /*!< \brief Total length. */
    u_short ip_id;          /*!< \brief Identification. */
    short   ip_off;         /*!< \brief Fragment offset field. */
    u_char  ip_ttl;         /*!< \brief Time to live. */
    u_char  ip_p;           /*!< \brief Protocol. */
    u_short ip_sum;         /*!< \brief Checksum. */
    u_long  ip_src;         /*!< \brief Source IP address. */
    u_long  ip_dst;         /*!< \brief Destination IP address. */
};

#define IPOPT_EOL       0       /*!< \brief End of option list. */
#define IPOPT_NOP       1       /*!< \brief No operation. */

#define IPOPT_RR        7       /*!< \brief Record packet route. */
#define IPOPT_TS        68      /*!< \brief Timestamp. */
#define IPOPT_SECURITY  130     /*!< \brief Provide s,c,h,tcc. */
#define IPOPT_LSRR      131     /*!< \brief Loose source route. */
#define IPOPT_SATID     136     /*!< \brief Satnet id. */
#define IPOPT_SSRR      137     /*!< \brief Strict source route. */

#define IPOPT_OPTVAL    0       /*!< \brief Option identifier offset. */
#define IPOPT_OLEN      1       /*!< \brief Option length offset. */
#define IPOPT_OFFSET    2       /*!< \brief Offset within option. */
#define IPOPT_MINOFF    4       /*!< \brief Minimum offset within option. */

#define MAXTTL      255     /*!< \brief Maximum time to live (seconds). */
#define IPDEFTTL    64      /*!< \brief Default time to live. */
#define IPFRAGTTL   60      /*!< \brief Time to live for fragments. */
#define IPTTLDEC    1       /*!< \brief Subtracted from time to live when forwarding. */

/*
 * API declarations.
 */
#include <dev/netbuf.h>
extern int slIpOutput(u_char proto, u_long dest, NETBUF *nb);

/*
 * Kernel declarations.
 */
#include <net/if_var.h>
extern void slIpInput(NUTDEVICE *dev, NETBUF *nb);

/*@}*/

#endif
