#ifndef _PRO_DHCP_H_
#define _PRO_DHCP_H_


#include <sys/types.h>

/*!
 * \file pro/dhcp.h
 * \brief DHCP protocol definitions.
 */

#define DHCP_SERVERPORT     67
#define DHCP_CLIENTPORT     68

#define DHCPOPT_PAD         0
#define DHCPOPT_NETMASK     1
#define DHCPOPT_GATEWAY     3
#define DHCPOPT_DNS         6
#define DHCPOPT_HOSTNAME    12
#define DHCPOPT_DOMAIN      15
#define DHCPOPT_BROADCAST   28
#define DHCPOPT_REQESTIP    50
#define DHCPOPT_LEASETIME   51
#define DHCPOPT_MSGTYPE     53
#define DHCPOPT_SID         54
#define DHCPOPT_RENEWALTIME 58
#define DHCPOPT_REBINDTIME  59
#define DHCPOPT_END         255

#define DHCP_DISCOVER   1
#define DHCP_OFFER      2
#define DHCP_REQUEST    3
#define DHCP_DECLINE    4
#define DHCP_ACK        5
#define DHCP_NAK        6
#define DHCP_RELEASE    7
#define DHCP_INFORM     8

/*!
 * \struct bootp dhcp.h pro/dhcp.h
 * \brief DHCP telegram structure.
 */
struct bootp {
    u_char  bp_op;              /*!< \brief Packet opcode type: 1=request, 2=reply */
    u_char  bp_htype;           /*!< \brief Hardware address type: 1=Ethernet */
    u_char  bp_hlen;            /*!< \brief Hardware address length: 6 for Ethernet */
    u_char  bp_hops;            /*!< \brief Gateway hops */
    u_long  bp_xid;             /*!< \brief Transaction ID */
    u_short bp_secs;            /*!< \brief Seconds since boot began */
    u_short bp_flags;           /*!< \brief RFC1532 broadcast, etc. */
    u_long  bp_ciaddr;          /*!< \brief Client IP address */
    u_long  bp_yiaddr;          /*!< \brief 'Your' IP address */
    u_long  bp_siaddr;          /*!< \brief Server IP address */
    u_long  bp_giaddr;          /*!< \brief Gateway IP address */
    u_char  bp_chaddr[16];      /*!< \brief Client hardware address */
    char    bp_sname[64];       /*!< \brief Server host name */
    char    bp_file[128];       /*!< \brief Boot file name */
    u_char  bp_options[312];    /*!< \brief Vendor-specific area */
};

extern int slDhcpClientStart(NUTDEVICE *dev);

#endif
