#ifndef _SYS_ATOM_H_
#define _SYS_ATOM_H_


#include <sys/types.h>

#ifdef __IMAGECRAFT__

#define AtomicInc(p)     (++(*p))
#define AtomicDec(p)     (--(*p))

#else

static inline void AtomicInc(volatile u_char *p)
{
    asm volatile(
        "in  __tmp_reg__, __SREG__" "\n\t"
        "cli"                       "\n\t"
        "ld r24, %a0"               "\n\t"
        "subi r24, lo8(-1)"         "\n\t"
        "st %a0, r24"               "\n\t"
        "out __SREG__, __tmp_reg__" "\n\t"
        :
        : "z" (p)
        : "r24"
    );
}

static inline void AtomicDec(volatile u_char *p)
{
    asm volatile(
        "in  __tmp_reg__, __SREG__" "\n\t"
        "cli"                       "\n\t"
        "ld r24, %a0"               "\n\t"
        "subi r24, lo8(1)"          "\n\t"
        "st %a0, r24"               "\n\t"
        "out __SREG__, __tmp_reg__" "\n\t"
        :
        : "z" (p)
        : "r24"
    );
}

#endif


#ifdef __IMAGECRAFT__

#define slEnterCritical()  \
{                           \
    asm("in R0, 0x3F\n"     \
        "push R0\n"         \
        "cli\n");           \
}

#define slExitCritical()   \
{                           \
    asm("pop R0\n"          \
        "out 0x3F, R0\n");  \
}

#else

static inline void slEnterCritical(void)
{
    asm volatile(
        "in  __tmp_reg__, __SREG__" "\n\t"
        "push __tmp_reg__"          "\n\t"
        "cli"                       "\n\t"
    );
}

static inline void slExitCritical(void)
{
    asm volatile(
        "pop __tmp_reg__"           "\n\t"
        "out __SREG__, __tmp_reg__" "\n\t"
    );
}

#endif


#endif
