#ifndef _SYS_HEAP_H_
#define _SYS_HEAP_H_


#include <stddef.h>
#include <sys/types.h>

/*!
 * \file sys/heap.h
 * \brief Heap management definitions.
 */

/*!
 * \struct _HEAPNODE heap.h sys/heap.h
 * \brief Heap memory node information structure.
 */
/*!
 * \typedef HEAPNODE
 * \brief Heap memory node type.
 */
typedef struct _HEAPNODE {
    u_short hn_size;            /*!< \brief Size of this node. */
    struct _HEAPNODE *hn_next;  /*!< \brief Link to next free node. */
} HEAPNODE;

extern HEAPNODE* volatile heapFreeList;


/*!
 * \brief Allocation threshold.
 *
 * Might be increased to avoid creating
 * too many small nodes.
 */
#define ALLOC_THRESHOLD 6

extern void *slHeapAlloc(u_short size);
extern void *slHeapAllocClear(u_short size);
extern int slHeapFree(void *block);
extern void slHeapAdd(void *addr, u_short size);
extern u_short slHeapAvailable(void);

#endif
