#ifndef _SYS_IFSTREAM_H_
#define _SYS_IFSTREAM_H_

/*!
 * \file sys/ifstream.h
 * \brief Stream device interface definitions.
 */

/*!
 * \brief Stream interface type.
 */
typedef struct _IFSTREAM IFSTREAM;

#include <sys/device.h>

/*!
 * \struct _IFSTREAM ifstream.h sys/ifstream.h
 * \brief Stream interface information structure.
 */
struct _IFSTREAM {
    int  (*if_input)(NUTDEVICE *);  /*!< \brief Wait for input. */
    int  (*if_output)(NUTDEVICE *); /*!< \brief Initiate output. */
    int  (*if_flush)(NUTDEVICE *);  /*!< \brief Wait until output buffer empty. */
    volatile u_char if_rx_idx;      /*!< \brief Next input index. */
    u_char if_rd_idx;               /*!< \brief Next read index. */
    volatile u_char if_tx_idx;      /*!< \brief Next output index. */
    u_char if_wr_idx;               /*!< \brief Next write index. */
    volatile u_char if_tx_act;      /*!< \brief Set if transmitter running. */
    u_char if_last_eol;             /*!< \brief Last end of line character read. */
    u_char if_rx_buf[256];          /*!< \brief Input buffer. */
    u_char if_tx_buf[256];          /*!< \brief Output buffer. */
};

extern int slIfStreamRead(NUTDEVICE *dev, char *data, int size);
extern int slIfStreamReadTran(NUTDEVICE *dev, char *data, int size);
extern int slIfStreamGetLine(NUTDEVICE *dev, char *data, int size);
extern int slIfStreamWrite(NUTDEVICE *dev, CONST char *data, int len);
extern int slIfStreamWriteTran(NUTDEVICE *dev, CONST char *data);
extern int slIfStreamWrite_P(NUTDEVICE *dev, PGM_P data, int len);
extern int slIfStreamFlush(NUTDEVICE *dev);

#endif
