#ifndef _SYS_PRINT_H_
#define _SYS_PRINT_H_



#include <pgmspace.h>

#include <sys/device.h>

/*!
 * \file sys/print.h
 * \brief Formatted printing definitions.
 */

/**
 * \name Format flags
 *
 * \brief Number printing formats.
 *
 * These flags are used as number printing formats. Several
 * flags may be combined using the binary OR operator.
 *
 * \see slPrintInteger
 *
 */
/*@{*/

#define FMTFLG_ZERO     0x01    /*!< \brief Set, if zero padding required */
#define FMTFLG_SIGNED   0x02    /*!< \brief Set, if signed value */
#define FMTFLG_PLUS     0x04    /*!< \brief Set to force sign */
#define FMTFLG_MINUS    0x08    /*!< \brief Set to force left justification */
#define FMTFLG_CAPITAL  0x10    /*!< \brief Set for capital letter digits */

/*@}*/

extern int slPrintBinary(NUTDEVICE *dev, CONST char *data, int len);
extern int slPrintBinary_P(NUTDEVICE *dev, PGM_P data, int len);
extern int slPrintString(NUTDEVICE *dev, CONST char *str);
extern int slPrintString_P(NUTDEVICE *dev, PGM_P str);
extern int slPrintInteger(NUTDEVICE *dev, u_long val, u_char radix, u_char width, u_char flags);
extern int slPrintFormat(NUTDEVICE *dev, CONST char *fmt, ...);
extern int slPrintFlush(NUTDEVICE *dev);

extern HANDLE app1, app2, app3, app4;

#endif
