#ifndef _SYS_SOCK_VAR_H_
#define _SYS_SOCK_VAR_H_



#include <sys/types.h>
#include <dev/netbuf.h>
#include <sys/device.h>

/*!
 * \file sys/sock_var.h
 * \brief UDP and TCP socket interface definitions.
 */

/*!
 * \brief UDP socket type.
 */
typedef struct udp_socket UDPSOCKET;

/*!
 * \struct udp_socket socket.h sys/sock_var.h
 * \brief UDP socket information structure.
 */
struct udp_socket {
    UDPSOCKET *so_next;     //!< Link to next tcp socket structure.
    u_short so_local_port;  //!< Local port number in net byte order.
    NETBUF  *so_rx_nb;      //!< Received, but not read by application.
    HANDLE  so_rx_rdy;      //!< Receiver event queue.
};

/*@}*/

extern void slUdpInput(NETBUF *nb);
extern int slUdpOutput(UDPSOCKET *sock, u_long dest, u_short port, NETBUF *nb);


/*********************************************************************\
 * TCP
\*********************************************************************/

/*!
 * \addtogroup xgTcpSocket
 */
/*@{*/

/*!
 * \brief TCP socket type.
 */
typedef struct tcp_socket TCPSOCKET;

/*!
 * \struct tcp_socket socket.h sys/sock_var.h
 * \brief TCP socket information structure.
 */
struct tcp_socket {
    TCPSOCKET *so_next;     //!< \brief Link to next tcp socket structure.
    volatile u_char  so_state;       //!< \brief Connection state, see tcp_fsm.h
    u_long  so_local_addr;  //!< \brief Local IP address in net byte order.
    u_short so_local_port;  //!< \brief Local port number in net byte order.
    u_long  so_remote_addr; //!< \brief Remote IP address in net byte order.
    u_short so_remote_port; //!< \brief Remote port number in net byte order.

    u_char  so_tx_flags;    //!< \brief Flags used during transmissions - see below
    u_long  so_tx_isn;      //!< \brief Initial sequence number.
    u_long  so_tx_una;      //!< \brief Unacknowledged sequence number.
    u_long  so_tx_nxt;      //!< \brief Next sequence number to send.
    u_long  so_tx_wl1;      //!< \brief Sequence number of last window update.
    u_long  so_tx_wl2;      //!< \brief Acknowledged sequence of last window update.
    u_short so_tx_win;      //!< \brief Peer's receive window.
    u_char  so_tx_dup;      //!< \brief Duplicate ACK counter.
    NETBUF  *so_tx_nbq;     //!< \brief Network buffers waiting to be acknowledged.
    HANDLE  so_tx_tq;       //!< \brief Threads waiting for transmit buffer space.

    u_long  so_rx_isn;      //!< \brief Initial sequence number of remote.
    u_long  so_rx_nxt;      //!< \brief Next sequence number to receive.
    u_short so_rx_win;      //!< \brief Local receive window.
    u_short so_rx_cnt;      //!< \brief Number of data bytes in the receive buffer.
    u_short so_rx_bsz;      //!< \brief Receive buffer size.
    u_char  *so_rx_buf;     //!< \brief Data waiting to be read by application.
    HANDLE  so_rx_tq;       //!< \brief Threads waiting for received data.
    NETBUF  *so_rx_nbq;     //!< \brief Network buffers received in advance.

    u_short so_mss;         //!< \brief MSS, limited by remote option or MTU.

    u_long  so_rto_next;    //!< \brief Next round-trip timeout
    u_long  so_srtt_avg;    //!< \brief Scaled round-trip timer average
    u_long  so_srtt_var;    //!< \brief Scaled round-trip timer variance
    u_short so_retransmits; //!< \brief Number of retransmits.
    u_short so_time_wait;   //!< \brief Time wait counter.
    u_short so_retran_time; //!< \brief Retransmit time counter.
    u_short so_last_error;  //!< \brief Last error code.
    HANDLE  so_pc_tq;       //!< \brief Listening thread.
    HANDLE  so_ac_tq;       //!< \brief Connecting thread.

    u_long  so_read_to;     //!< \brief Read timeout.
    u_long  so_write_to;    //!< \brief Write timeout.
    u_long  so_oos_drop;    //!< \brief Out of sequence dropped
};

/*
 * TCP send flags.
 */
#define SO_FIN      0x01    //!< \brief Send FIN after all data has been transmitted.
#define SO_SYN      0x02    //!< \brief Send SYN first.
#define SO_FORCE    0x08    //!< \brief Force sending ACK.
#define SO_ACK      0x10    //!< \brief Send ACK

/*@}*/

#include <netinet/tcp_fsm.h>

extern int slTcpOutput(TCPSOCKET *sock, u_char *data, u_short size);
extern int slTcpReject(NETBUF *nb);

#endif
