#ifndef _SYS_SOCKET_H_
#define _SYS_SOCKET_H_

/*!
 * \file sys/socket.h
 * \brief UDP and TCP socket interface definitions.
 */

/*!
 * \addtogroup xgSocket
 */



#include <sys/sock_var.h>

/*
 * Types
 */
#define SOCK_STREAM 1       /*!< \brief Stream socket */
#define SOCK_DGRAM  2       /*!< \brief Datagram socket */
#define SOCK_RAW    3       /*!< \brief Raw-protocol interface */

/*
 * Option flags per-socket.
 */
#define SO_DEBUG        0x0001      /*!< \brief Turn on debugging info recording */
#define SO_ACCEPTCONN   0x0002      /*!< \brief Socket has had listen() */
#define SO_REUSEADDR    0x0004      /*!< \brief Allow local address reuse */
#define SO_KEEPALIVE    0x0008      /*!< \brief Keep connections alive */
#define SO_DONTROUTE    0x0010      /*!< \brief Just use interface addresses */
#define SO_BROADCAST    0x0020      /*!< \brief Permit sending of broadcast msgs */
#define SO_USELOOPBACK  0x0040      /*!< \brief Bypass hardware when possible */
#define SO_LINGER       0x0080      /*!< \brief Linger on close if data present */
#define SO_OOBINLINE    0x0100      /*!< \brief Leave received OOB data in line */
#define SO_REUSEPORT    0x0200      /*!< \brief Allow local address & port reuse */

/*
 * Additional options, not kept in so_options.
 */
#define SO_SNDBUF       0x1001      /*!< \brief Send buffer size */
#define SO_RCVBUF       0x1002      /*!< \brief Receive buffer size */
#define SO_SNDLOWAT     0x1003      /*!< \brief Send low-water mark */
#define SO_RCVLOWAT     0x1004      /*!< \brief Receive low-water mark */
#define SO_SNDTIMEO     0x1005      /*!< \brief Send timeout */
#define SO_RCVTIMEO     0x1006      /*!< \brief Receive timeout */
#define SO_ERROR        0x1007      /*!< \brief Get error status and clear */
#define SO_TYPE         0x1008      /*!< \brief Get socket type */

/*
 * Address families.
 */
#define AF_INET     2       /*!< \brief internetwork: UDP, TCP, etc. */

extern TCPSOCKET *slTcpCreateSocket(void);
extern int slTcpSetSockOpt(TCPSOCKET *sock, int optname, CONST void *optval, int optlen);
extern int slTcpGetSockOpt(TCPSOCKET *sock, int optname, void *optval, int optlen);
extern int slTcpConnect(TCPSOCKET *sock, u_long addr, u_short port);
extern int slTcpAccept(TCPSOCKET *sock, u_short port);
extern void slTcpInput(NETBUF *nb);
extern int slTcpSend(TCPSOCKET *sock, void *data, u_short len);
extern int slTcpCloseSocket(TCPSOCKET *sock);
extern void slTcpDestroySocket(TCPSOCKET *sock);
extern int slTcpReceive(TCPSOCKET *sock, void *data, u_short size);
extern TCPSOCKET *slTcpFindSocket(u_short lport, u_short rport, u_long raddr);
extern int slTcpError(TCPSOCKET *sock);

extern UDPSOCKET *slUdpCreateSocket(u_short port);
extern int slUdpSendTo(UDPSOCKET *sock, u_long addr, u_short port, void *data, u_short len);
extern int slUdpReceiveFrom(UDPSOCKET *sock, u_long *addr, u_short *port, void *data, u_short size, u_long timeout);
extern int slUdpDestroySocket(UDPSOCKET *sock);
extern UDPSOCKET *slUdpFindSocket(u_short port);

#endif
