#ifndef _SYS_THREAD_H_
#define _SYS_THREAD_H_



#include <sys/device.h>

/*!
 * \file sys/thread.h
 * \brief Thread management definitions.
 */

#define DEADBEEF    0xDEADBEEF


/*!
 * \addtogroup xgThread
 */
/*@{*/

/*!
 * Thread information structure type.
 */
typedef struct _NUTTHREADINFO NUTTHREADINFO;

/*!
 * \struct _NUTTHREADINFO thread.h sys/thread.h
 * \brief Thread information structure.
 */
struct _NUTTHREADINFO {
    NUTTHREADINFO *volatile td_next;    /*!< \brief Linked list of all threads. */
    NUTTHREADINFO *td_qnxt;    /*!< \brief Linked list of all queued thread. */
    u_char  td_name[9];     /*!< \brief Name of this thread. */
    u_char  td_state;       /*!< \brief Operating state. One of TDS_ */
    u_short td_sp;          /*!< \brief Stack pointer. */
    u_char  td_priority;    /*!< \brief Priority level. 0 is highest priority. */
    u_char *td_memory;     /*!< \brief Pointer to heap memory used for stack. */
    HANDLE  td_timer;       /*!< \brief Event timer. */
    HANDLE  td_queue;       /*!< \brief Root entry of the waiting queue. */
};
/*@}*/

/*!
 * \addtogroup xgThreadState
 */
/*@{*/
#define TDS_TERM        0   /*!< Thread has exited. */
#define TDS_RUNNING     1   /*!< Thread is running. */
#define TDS_READY       2   /*!< Thread is ready to run. */
#define TDS_SLEEP       3   /*!< Thread is sleeping. */
/*@}*/


extern NUTTHREADINFO* volatile runningThread;
extern NUTTHREADINFO* volatile nutThreadList;
extern NUTTHREADINFO * volatile runQueue;



extern HANDLE slThreadCreate(u_char *name, void (*fn)(void *), void *arg, u_short stackSize);
extern u_char slThreadSetPriority(u_char level);

extern void slThreadResumeAsync(HANDLE th);
extern void slThreadWake(HANDLE timer, HANDLE th);
extern void slThreadYield(void);

extern void slThreadAddPriQueue(NUTTHREADINFO *td, NUTTHREADINFO **tqpp);
extern void slThreadRemoveQueue(NUTTHREADINFO *td, NUTTHREADINFO *volatile *tqpp);

extern void slThreadSwitch(void);

/*!
 * \brief Macro for thread entry definitions.
 */
#define THREAD(threadfn, arg) \
void threadfn(void *arg) __attribute__ ((noreturn)); \
void threadfn(void *arg)

extern void DumpThreads(HANDLE handle);

#endif
