#ifndef _SYS_TIMER_H
#define _SYS_TIMER_H


#include <sys/device.h>

/*!
 * \file sys/timer.h
 * \brief Timer management definitions.
 */

/*!
 * \brief Timer type.
 */
typedef struct _NUTTIMERINFO NUTTIMERINFO;

/*!
 * \struct _NUTTIMERINFO timer.h sys/timer.h
 * \brief Timer information structure.
 */
struct _NUTTIMERINFO {
    /*! \brief Link to next timer. 
     */
    NUTTIMERINFO *tn_next;          
    /*! \brief Number of system ticks. 
     *  Set to zero on one-shot timers.
     */
    u_long tn_ticks;        
    /*! \brief Decremented by one on each system tick intervall. 
     */
    u_long tn_ticks_left;   
    /*! \brief Callback function. 
     */
    void (*tn_callback)(HANDLE, void *);    
    /*! \brief Argument pointer passed to callback function. 
     */
    volatile void *tn_arg;          
};

extern NUTTIMERINFO* volatile nutTimerList;
extern NUTTIMERINFO* volatile nutTimerPool;

extern void slTimerInit(void);


/*
 * API declarations.
 */

#define TM_ONESHOT  0x01

extern HANDLE slTimerStart(u_long ms, void (*callback)(HANDLE, void *), void *arg, u_char flags);
extern void slTimerStop(HANDLE handle);
extern void slTimerStopAsync(HANDLE handle);
extern void slSleep(u_long ms);
extern void slDelay(u_char ms);

extern u_long slGetCpuClock(void);

#endif
