#ifndef _SYS_TYPES_H_
#define _SYS_TYPES_H_


#include <compiler.h>

/*!
 * \file sys/types.h
 * \brief sl/OS type declarations.
 */

#define __SFR_OFFSET 0 
#define SFR_IO_ADDR(sfr) ((sfr) - __SFR_OFFSET)
#define SFR_MEM_ADDR(sfr) (sfr)
#define SFR_IO_REG_P(sfr) ((sfr) < 0x40 + __SFR_OFFSET) 


/*! \brief Unsigned 8-bit value. */
typedef unsigned char      u_char;
/*! \brief Unsigned 16-bit value. */
typedef unsigned short     u_short;
/*! \brief Unsigned 16-bit value. */
typedef unsigned int       u_int;
/*! \brief Unsigned 32-bit value */
typedef unsigned long      u_long;
/*! \brief Unsigned 64-bit value */
typedef unsigned long long u_longlong;
/*! \brief Void pointer */
typedef void * HANDLE;

#ifdef __IMAGECRAFT__

#define __byte_swap2(val)           \
    (((val & 0xff) << 8) |          \
     ((val & 0xff00) >> 8))

#define __byte_swap4(val)           \
    (((val & 0xff) << 24) |         \
     ((val & 0xff00) << 8) |        \
     ((val & 0xff0000) >> 8) |      \
     ((val & 0xff000000) >> 24))


#else

/*
 * Conversion of 16 bit value to network order.
 */
static inline u_short __byte_swap2(u_short val)
{
    asm volatile(
        "mov __tmp_reg__, %A0\n\t"
        "mov %A0, %B0\n\t"
        "mov %B0, __tmp_reg__\n\t"
        : "=r" (val)
        : "0" (val)
    );
    return val;
}

/*
 * Conversion of 32 bit value to network order.
 */
static inline u_long __byte_swap4(u_long val)
{
    asm volatile(
        "mov __tmp_reg__, %A0\n\t"
        "mov %A0, %D0\n\t"
        "mov %D0, __tmp_reg__\n\t"
        "mov __tmp_reg__, %B0\n\t"
        "mov %B0, %C0\n\t"
        "mov %C0, __tmp_reg__\n\t"
        : "=r" (val)
        : "0" (val)
    );
    return val;
}

#endif

/*!
 * \brief Convert short value from host to network byte order.
 */
#define htons(x) __byte_swap2(x)

/*!
 * \brief Convert long value from host to network byte order.
 */
#define htonl(x) __byte_swap4(x)

/*!
 * \brief Convert short value from network to host byte order.
 */
#define ntohs(x) __byte_swap2(x)

/*!
 * \brief Convert long value from network to host byte order.
 */
#define ntohl(x) __byte_swap4(x)

/*@}*/

#endif
