
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <net/ether.h>


/*!
 * \brief Handle incoming Ethernet frames.
 *
 * Splits the Ethernet frame into the data link and the
 * network part. Then the frame is routed to the proper
 * handler, based on the type field in the Ethernet header.
 *
 * If the frame neither contains an IP nor an ARP type
 * telegram, then it is silently discarded.
 *
 * \note This routine is called by the device driver on
 *       incoming ethernet packets. Applications typically do
 *       not call this function.
 *
 * \param dev Identifies the device that received the frame.
 * \param nb  Pointer to a network buffer structure containing 
 *            the ethernet frame.
 */
void slEtherInput(NUTDEVICE *dev, NETBUF *nb)
{
    ETHERHDR *eh;

    /*
     * Split the Ethernet frame.
     */
    eh = (ETHERHDR *)nb->nb_dl.vp;
    nb->nb_nw.vp = eh + 1;
    nb->nb_nw.sz = nb->nb_dl.sz - sizeof(ETHERHDR);
    nb->nb_dl.sz = sizeof(ETHERHDR);

    /*
     * Route frame to the correct handler.
     */
    switch(ntohs(eh->ether_type)) {
    case ETHERTYPE_IP:
        slIpInput(dev, nb);
        break;
    case ETHERTYPE_ARP:
        slArpInput(dev, nb);
        break;
    default:
        slNetBufFree(nb);
        break;
    }
}

/*@}*/
