
#include <string.h>

#include <netinet/if_ether.h>
#include <net/ether.h>

/*!
 * \brief Send Ethernet frame.
 *
 * Send an Ethernet frame of a given type using the specified device.
 *
 * \param dev   Identifies the device to use.
 * \param type  Type of this frame, either ETHERTYPE_IP or ETHERTYPE_ARP.
 * \param ha    MAC address of the destination, set null for broadcasts.
 * \param nb    Network buffer structure containing the packet to be sent.
 *              The structure must have been allocated by a previous
 *              call slNetBufAlloc().
 *
 * \return 0 on success, -1 in case of any errors.
 */
int slEtherOutput(NUTDEVICE *dev, u_short type, u_char *ha, NETBUF *nb)
{
    ETHERHDR *eh;
    IFNET *nif;

    if(slNetBufAlloc(nb, NBAF_DATALINK, sizeof(ETHERHDR)) == 0)
        return -1;

    eh = (ETHERHDR *)nb->nb_dl.vp;
    nif = dev->dev_icb;
    memcpy(eh->ether_shost, nif->if_mac, 6);
    if(ha)
        memcpy(eh->ether_dhost, ha, 6);
    else
        memset(eh->ether_dhost, 0xff, 6);
    eh->ether_type = htons(type);

    /*
     * Call the network device output routine.
     */
    return (*nif->if_send)(dev, nb);
}

/*@}*/
