
#include <netinet/ip_icmp.h>
#include <netinet/ipcsum.h>
#include <netinet/icmp.h>

/*
 * \param dev Identifies the device to be used. This pointer must 
 *            have been retrieved by using the slDeviceOpen() function.
 */
static int slIcmpReflect(NUTDEVICE *dev, u_char type, NETBUF *nb)
{
    IPHDR *ip;
    u_long dest;
    IFNET *nif;

    ip = nb->nb_nw.vp;
    dest = ip->ip_src;
    nif = dev->dev_icb;
    ip->ip_src = nif->if_local_ip;
    ip->ip_ttl = MAXTTL;

    return slIcmpOutput(type, dest, nb);
}


/*!
 * \brief Handle incoming ICMP packets.
 *
 * \note This routine is called by the IP layer on incoming 
 *       ICMP datagrams. Applications typically do not call 
 *       this function.
 *
 * \param dev Identifies the device that received the packet.
 * \param nb  Pointer to a network buffer structure containing 
 *            the ICMP datagram.
 */
void slIcmpInput(NUTDEVICE *dev, NETBUF *nb)
{
    /*
     * Silently discard packets, which are too small.
     */
    if(nb->nb_tp.sz > ICMP_MINLEN) {
        ICMPHDR *icp = (ICMPHDR *)nb->nb_tp.vp;
        if(icp->icmp_type == ICMP_ECHO)
            slIcmpReflect(dev, ICMP_ECHOREPLY, nb);
    }
    slNetBufFree(nb);
}

/*@}*/
