
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/icmp.h>
#include <netinet/ip_icmp.h>
#include <netinet/ipcsum.h>
#include <net/netdebug.h>
#include <sys/socket.h>

#include <sys/print.h>

extern TCPSOCKET *tcpSocketList;
extern UDPSOCKET *udpSocketList;

char tcp_trace = 0;

void slKDumpTcpHeader(u_char *ds, TCPSOCKET *sock, NETBUF *nb)
{
    TCPHDR *th = (TCPHDR *)nb->nb_tp.vp;

    slPrintFormat(0, "%s%04X[%u]-SEQ(%lx)", ds, sock, nb->nb_ap.sz, ntohl(th->th_seq));
    if(th->th_flags & TH_ACK) 
        slPrintFormat(0, "-ACK(%lx)", ntohl(th->th_ack));
    if(th->th_flags & TH_FIN)
        slPrintString(0, "-FIN");
    if(th->th_flags & TH_SYN)
        slPrintString(0, "-SYN");
    if(th->th_flags & TH_RST)
        slPrintString(0, "-RST");
    if(th->th_flags & TH_PUSH)
        slPrintString(0, "-PSH");
    if(th->th_flags & TH_URG)
        slPrintString(0, "-URG");
    slPrintString(0, "\r\n");
}

void slKDumpSockState(u_char state, u_char *lead, u_char *trail)
{
    if(lead)
        slPrintString(0, lead);
    switch(state) {
    case TCPS_LISTEN:
        slPrintString(0, "LISTEN");
        break;
    case TCPS_SYN_SENT:
        slPrintString(0, "SYNSENT");
        break;
    case TCPS_SYN_RECEIVED:
        slPrintString(0, "SYNRCVD");
        break;
    case TCPS_ESTABLISHED:
        slPrintString(0, "ESTABL");
        break;
    case TCPS_FIN_WAIT_1:
        slPrintString(0, "FINWAIT1");
        break;
    case TCPS_FIN_WAIT_2:
        slPrintString(0, "FINWAIT2");
        break;
    case TCPS_CLOSE_WAIT:
        slPrintString(0, "CLOSEWAIT");
        break;
    case TCPS_CLOSING:
        slPrintString(0, "CLOSING");
        break;
    case TCPS_LAST_ACK:
        slPrintString(0, "LASTACK");
        break;
    case TCPS_TIME_WAIT:
        slPrintString(0, "TIMEWAIT");
        break;
    case TCPS_CLOSED:
        slPrintString(0, "CLOSED");
        break;
    default:
        slPrintString(0, "?UNK?");
        break;
    }
    if(trail)
        slPrintString(0, trail);
}


void slKDumpSocketList(void)
{
    TCPSOCKET *ts;
    UDPSOCKET *us;

    slPrintString(0, "\r\nSock Typ Local                  Remote                 State\r\n");
    /*                   1234 123 123456789012345:123456 123456789012345:123456 */
    for(ts = tcpSocketList; ts; ts = ts->so_next) {
        slPrintFormat(0, "%04X TCP %15s:%-6u ", ts, 
                        inet_ntoa(ts->so_local_addr), ntohs(ts->so_local_port));
        slPrintFormat(0, "%15s:%-6u ", inet_ntoa(ts->so_remote_addr), 
                                      ntohs(ts->so_remote_port));
        slKDumpSockState(ts->so_state, 0, "\r\n");
    }
    for(us = udpSocketList; us; us = us->so_next) {
        slPrintFormat(0, "%04X UDP %6u\r\n", us, ntohs(us->so_local_port));
    }
}


/*!
 * \brief Switch on/off TCP trace.
 */
void slKTcpTrace(char enable)
{
    tcp_trace = enable;
}

