
#include <sys/thread.h>

#include <netinet/ip.h>
#include <sys/socket.h>
#include <netinet/tcp.h>



/*!
 * \brief Process incoming TCP segments from IP layer.
 *
 * \warn The caller must take care not to pass broadcast
 *       or multicast segments.
 *
 * \note This routine is called by the IP layer on incoming 
 *       TCP  segments. Applications typically do not call 
 *       this function.
 *
 */
void slTcpInput(NETBUF *nb)
{
    TCPHDR *th = (TCPHDR *)nb->nb_tp.vp;

    nb->nb_ap.sz = nb->nb_tp.sz - (th->th_off * 4);
    if(nb->nb_ap.sz)
        nb->nb_ap.vp = ((u_long *)th) + th->th_off;
    nb->nb_tp.sz = th->th_off * 4;

    /*
     * According to RFC1122 we MUST check the checksum
     * on incoming segments. Anyway, we rely on lower
     * level checksums to save processing resources.
     *
     * However, we may combine checksum calculation
     * with moving data to the receiver buffer.
     */

    slTcpStateMachine(nb);
}

/*@}*/
