
#include <sys/event.h>

#include <netinet/udp.h>
#include <sys/socket.h>

/*!
 * \brief Handle incoming UDP packets.
 *
 * \note This routine is called by the IP layer on
 *       incoming UDP packets. Applications typically do
 *       not call this function.
 *
 * \param nb  Network buffer structure containing the UDP packet.
 */
void slUdpInput(NETBUF *nb)
{
    UDPHDR *uh;
    UDPSOCKET *sock;

    uh = (UDPHDR *)nb->nb_tp.vp;

    nb->nb_ap.vp = uh + 1;
    nb->nb_ap.sz = nb->nb_tp.sz - sizeof(UDPHDR);
    nb->nb_tp.sz = sizeof(UDPHDR);

    /*
     * Find a port. If none exists, return an ICMP unreachable.
     */
    if((sock = slUdpFindSocket(uh->uh_dport)) == 0) {
        slNetBufFree(nb);
        return;
    }

    /*
     * If we stored a previous buffer, remove
     * it and store the new one.
     */
    if(sock->so_rx_nb)
        slNetBufFree(sock->so_rx_nb);
    sock->so_rx_nb = nb;

    slEventPost(&sock->so_rx_rdy);
}

/*@}*/
