
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/ipcsum.h>
#include <net/route.h>
#include <sys/socket.h>



/*!
 * \brief Send an UDP packet.
 *
 * \note Applications typically do not call this function but 
 *       use the UDP socket interface.
 *
 * \return 0 on success, -1 otherwise.
 */
int slUdpOutput(UDPSOCKET *sock, u_long daddr, u_short port, NETBUF *nb)
{
    u_long saddr;
    u_short csum;
    UDPHDR *uh;
    NUTDEVICE *dev;
    IFNET *nif;

    if((nb = slNetBufAlloc(nb, NBAF_TRANSPORT, sizeof(UDPHDR))) == 0)
        return -1;

    uh = nb->nb_tp.vp;
    uh->uh_sport = sock->so_local_port;
    uh->uh_dport = htons(port);
    uh->uh_ulen = htons(nb->nb_tp.sz + nb->nb_ap.sz);

    /*
     * Get local address for this destination.
     */
    if((dev = slIpRouteQuery(daddr, &saddr)) != 0) {
        nif = dev->dev_icb;
        saddr = nif->if_local_ip;
    }
    else
        saddr = 0;

    uh->uh_sum = 0;
    csum = slIpPseudoChkSumPartial(saddr, daddr, IPPROTO_UDP, uh->uh_ulen);
    csum = slIpChkSumPartial(csum, uh, sizeof(UDPHDR));
    uh->uh_sum = slIpChkSum(csum, nb->nb_ap.vp, nb->nb_ap.sz);
    
    return slIpOutput(IPPROTO_UDP, daddr, nb);
}

/*@}*/
