
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/event.h>
#include <sys/heap.h>
#include <sys/atom.h>
#include <sys/print.h>

char os_trace = 0;
char heap_trace = 0;

static char *states[] = { "TRM", "RUN", "RDY", "SLP" };

void slKDumpThreadQueue(NUTTHREADINFO *tdp)
{
    slEnterCritical();
    slPrintString_P(0, PSTR("\r\nHndl Name     Prio Sta QUE  Timr StkP FreeMem\r\n"));
    /*                          1234 12345678 1234 123 1234 1234 1234 12345 */
    if(tdp == SIGNALED) {
        slPrintString_P(0, PSTR("SIGNALED\r\n"));
        slExitCritical();
        return;
    }
    while(tdp) {
        slPrintFormat(0, "%04X %-8s %4u %s %04X %04X %04X %5u %s\r\n", 
                        tdp, tdp->td_name, tdp->td_priority,
                        states[tdp->td_state], tdp->td_queue, tdp->td_timer,
                        tdp->td_sp, (u_short)tdp->td_sp - (u_short)tdp->td_memory,
                        *((u_long *)tdp->td_memory) != DEADBEEF &&
                        *((u_long *)(tdp->td_memory + 4)) != DEADBEEF &&
                        *((u_long *)(tdp->td_memory + 8)) != DEADBEEF &&
                        *((u_long *)(tdp->td_memory + 12)) != DEADBEEF 
                        ? "Corr" : "OK");
        tdp = tdp->td_qnxt;
    }
    slExitCritical();
}

void slKDumpThreadList(void)
{
    NUTTHREADINFO *tqp;
    NUTTHREADINFO *tdp;

    slEnterCritical();
    tdp = nutThreadList;
    slPrintString_P(0, PSTR("\r\nHndl Name     Prio Sta Que  Timr StkP FreeMem\r\n"));
    /*                   1234 12345678 1234 123 1234 1234 1234 12345 */
    while(tdp) {
        slPrintFormat(0, "%04X %-8s %4u %s %04X %04X %04X %5u %s", 
                        tdp, tdp->td_name, tdp->td_priority,
                        states[tdp->td_state], tdp->td_queue, tdp->td_timer,
                        tdp->td_sp, (u_short)tdp->td_sp - (u_short)tdp->td_memory,
                        *((u_long *)tdp->td_memory) != DEADBEEF ? "CR" : "OK");
        if(tdp->td_queue) {
            tqp = *(NUTTHREADINFO **)(tdp->td_queue);
            if(tqp == SIGNALED)
                slPrintString_P(0, PSTR("SIGNALED"));
            else {
                while(tqp) {
                    slPrintFormat(0, " %04X", tqp);
                    tqp = tqp->td_qnxt;
                }
            }
        }
        slPrintString(0, "\r\n");
        tdp = tdp->td_next;
    }
    slExitCritical();
}

void slKDumpTimerList(void)
{
    NUTTIMERINFO *tnp;

    slEnterCritical();
    if((tnp = nutTimerList) != 0) {
        slPrintString_P(0, PSTR("Addr Ticks  Left Callback\r\n"));
        while(tnp) {
            slPrintFormat(0, "%04X%6lu%6lu ", tnp, tnp->tn_ticks, tnp->tn_ticks_left);
            if(tnp->tn_callback == slThreadWake)
                slPrintString_P(0, PSTR("slThreadWake"));
            else if(tnp->tn_callback == slEventTimeout)
                slPrintString_P(0, PSTR("slEventTimeout"));
            else
                slPrintFormat(0, "%05lX", (u_long)((u_short)tnp->tn_callback) << 1);
            slPrintFormat(0, "(%04X)\r\n", tnp->tn_arg);
            tnp = tnp->tn_next;
        }
    }
    if((tnp = nutTimerPool) != 0) {
        slPrintString_P(0, PSTR("Pool:"));
        while(tnp) {
            slPrintFormat(0, " %04X", tnp);
            tnp = tnp->tn_next;
        }
        slPrintString(0, "\r\n");
    }
    slExitCritical();
}

/*!
 * \brief Switch on/off OS trace.
 */
void slKOsTrace(char enable)
{
    os_trace = enable;
}

void slKDumpHeap(void)
{
    HEAPNODE *node;
    u_short sum = 0;
    u_short avail;

    slPrintString(0, "\r\n");
    for(node = heapFreeList; node; node = node->hn_next) {
        sum += node->hn_size;
        slPrintFormat(0, "%04x %5d\r\n", (u_short)node, node->hn_size);
        if((u_short)node < 0x60 || (u_short)node > 0x7fff)
            break;
    }
    if((avail = slHeapAvailable()) != sum)
        slPrintFormat(0, "%u counted, but %u reported\r\n", sum, avail);
    else
        slPrintFormat(0, "%u bytes free\r\n", avail);
}


/*!
 * \brief Switch on/off dynamic memory trace.
 */
void slKHeapTrace(char enable)
{
    heap_trace = enable;
}



