
#include <string.h>
#include <sys/heap.h>
#include <sys/print.h>

#include "dencode.h"
#include <pro/httpd.h>


AUTHINFO *authList = 0;

/*!
 * \brief Look up an authorization entry.
 */
static AUTHINFO *slHttpAuthLookup(CONST char *dirname, CONST char *login)
{
    AUTHINFO *auth;

    for(auth = authList; auth; auth = auth->auth_next) {
        if(dirname && strcmp(dirname, auth->auth_dirname))
            continue;
        if(login && strcmp(login, auth->auth_login))
            continue;
        break;
    }
    return auth;
}

/*!
 * \brief Register an authorization entry.
 *
 * Protect a specified directory from unauthorized access.
 *
 * \warn Directories not registered by this function are
 *       accessible by anyone.
 *
 * \param dirname Name of the directory to protect.
 * \param login   Required login to access this directory. This
 *                string must contain a user name, followed by
 *                a colon followed by an uncrypted password.
 *
 * \return 0 on success, -1 otherwise.
 */
int slRegisterAuth(CONST char *dirname, CONST char *login)
{
    AUTHINFO *auth;

    if((auth = slHeapAlloc(sizeof(AUTHINFO))) == 0) 
        return -1;
    auth->auth_next = authList;
    auth->auth_dirname = dirname;
    auth->auth_login = login;
    authList = auth;

    return 0;
}

/*!
 * \brief Validate an authorization request.
 *
 * \note This function is automatically called by the HTTP
 *       library on incoming requests. Applications do not
 *       need to call this function.
 *
 * \param req Request to be checked.
 *
 * \return 0, if access granted, -1 otherwise.
 */
int slHttpAuthValidate(REQUEST *req)
{
    char *realm;
    char *cp = 0;
    int   rc = -1;

    /*
     * Get directory by chopping off filename.
     */
    realm = req->req_url;
    if((cp = strrchr(realm, '/')) != 0)
        *cp = 0;
    else
        realm = ".";

    /*
     * Check if authorization required.
     */
    if(slHttpAuthLookup(realm, 0)) {
        /*
         * Check authorization.
         */
        if(req->req_auth) {
            /*
             * Acceptint basic authorization only.
             */
            if(strncmp(req->req_auth, "Basic ", 6) == 0) {
                slDecodeBase64(req->req_auth + 6);
                if(slHttpAuthLookup(realm, req->req_auth + 6))
                    rc = 0;
            }
        }
    }
    else
        rc = 0;

    if(cp)
        *cp = '/';

    return rc;
}

/*@}*/
