
#include <string.h>
#include <sys/heap.h>
#include <sys/print.h>
extern NUTDEVICE *uart;

#include <pro/httpd.h>


CGIFUNCTION * volatile cgiFunctionList = 0;

/*!
 * \brief Register a CGI function.
 *
 * \bug Duplicate names not checked.
 *
 * \param name Name of this CGI function.
 * \param func The function to be called, if the
 *             client requests the specified name.
 *
 * \return 0 on success, -1 otherwise.
 */
int slRegisterCgi(char *name, int (*func)(NUTDEVICE *, REQUEST *))
{
    CGIFUNCTION *cgi;

    if((cgi = slHeapAlloc(sizeof(CGIFUNCTION))) == 0)
        return -1;
    cgi->cgi_next = cgiFunctionList;
    cgi->cgi_name = name;
    cgi->cgi_func = func;
    cgiFunctionList = cgi;

    return 0;
}

/*!
 * \brief Process an incoming CGI request.
 *
 * Applications do not need to call this function. It
 * is automatically called by slHttpProcessRequest().
 *
 * \param sostream Socket stream of the connection, previously
 *                 created by calling slSoStreamCreate().
 * \param req      Contains the HTTP request.
 */
void slCgiProcessRequest(NUTDEVICE *sostream, REQUEST *req)
{
    CGIFUNCTION *cgi;

    if(req->req_method != METHOD_GET && req->req_method != METHOD_POST) {
        slHttpSendError(sostream, req, 501);
        return;
    }
    for(cgi = cgiFunctionList; cgi; cgi = cgi->cgi_next) {
        if(strcasecmp(cgi->cgi_name, req->req_url + 8) == 0)
            break;
    }
    if(cgi == 0)
        slHttpSendError(sostream, req, 404);
    else if((*cgi->cgi_func)(sostream, req))
        slHttpSendError(sostream, req, 500);
    return;
}

/*@}*/
